@@ -4,14 +4,15 @@ utils::globalVariables(c("measurementEstimate","transitionEstimate", "C",
 
 ges <- function(data, orders=c(1,1), lags=c(1,frequency(data)),
                 persistence=NULL, transition=NULL, measurement=NULL,
-                initial=c("optimal","backcasting"),
+                initial=c("optimal","backcasting"), ic=c("AICc","AIC","BIC"),
                 cfType=c("MSE","MAE","HAM","MLSTFE","MSTFE","MSEh"),
                 h=10, holdout=FALSE,
                 intervals=c("none","parametric","semiparametric","nonparametric"), level=0.95,
                 intermittent=c("none","auto","fixed","croston","tsb","sba"),
                 bounds=c("admissible","none"),
                 silent=c("none","all","graph","legend","output"),
-                xreg=NULL, initialX=NULL, updateX=FALSE, persistenceX=NULL, transitionX=NULL, ...){
+                xreg=NULL, xregDo=c("use","select"), initialX=NULL,
+                updateX=FALSE, persistenceX=NULL, transitionX=NULL, ...){
 # General Exponential Smoothing function. Crazy thing...
 #
 #    Copyright (C) 2016  Ivan Svetunkov
@@ -58,100 +59,40 @@ ges <- function(data, orders=c(1,1), lags=c(1,frequency(data)),
     environment(ssInput) <- environment();
     ssInput(modelType="ges",ParentEnvironment=environment());
 
-##### Preset y.fit, y.for, errors and basic parameters #####
-    matvt <- matrix(NA,nrow=obsStates,ncol=n.components);
-    y.fit <- rep(NA,obsInsample);
-    y.for <- rep(NA,h);
-    errors <- rep(NA,obsInsample);
-
-##### Prepare exogenous variables #####
-    xregdata <- ssXreg(data=data, xreg=xreg, updateX=updateX,
-                       persistenceX=persistenceX, transitionX=transitionX, initialX=initialX,
-                       obsInsample=obsInsample, obsAll=obsAll, obsStates=obsStates, maxlag=maxlag, h=h, silent=silentText);
-    n.exovars <- xregdata$n.exovars;
-    matxt <- xregdata$matxt;
-    matat <- xregdata$matat;
-    matFX <- xregdata$matFX;
-    vecgX <- xregdata$vecgX;
-    xreg <- xregdata$xreg;
-    xregEstimate <- xregdata$xregEstimate;
-    FXEstimate <- xregdata$FXEstimate;
-    gXEstimate <- xregdata$gXEstimate;
-    initialXEstimate <- xregdata$initialXEstimate;
-    xregNames <- colnames(matat);
-
-# These three are needed in order to use ssgeneralfun.cpp functions
-    Etype <- "A";
-    Ttype <- "N";
-    Stype <- "N";
-
-# Check number of parameters vs data
-    n.param.exo <- FXEstimate*length(matFX) + gXEstimate*nrow(vecgX) + initialXEstimate*ncol(matat);
-    n.param.max <- n.param.max + n.param.exo + (intermittent!="n");
-
-##### Check number of observations vs number of max parameters #####
-    if(obsNonzero <= n.param.max){
-        if(!silentText){
-            message(paste0("Number of non-zero observations is ",obsNonzero,
-                           ", while the number of parameters to estimate is ", n.param.max,"."));
-        }
-        stop("Not enough observations. Can't fit the model you ask.",call.=FALSE);
-    }
-
-##### Preset values of matvt ######
-    slope <- cov(yot[1:min(12,obsNonzero),],c(1:min(12,obsNonzero)))/var(c(1:min(12,obsNonzero)));
-    intercept <- sum(yot[1:min(12,obsNonzero),])/min(12,obsNonzero) - slope * (sum(c(1:min(12,obsNonzero)))/min(12,obsNonzero) - 1);
-
-    vtvalues <- intercept;
-    if((orders %*% lags)>1){
-        vtvalues <- c(vtvalues,slope);
-    }
-    if((orders %*% lags)>2){
-        vtvalues <- c(vtvalues,yot[1:(orders %*% lags-2),]);
-    }
-
-    vt <- matrix(NA,maxlag,n.components);
-    for(i in 1:n.components){
-        vt[(maxlag - modellags + 1)[i]:maxlag,i] <- vtvalues[((cumsum(c(0,modellags))[i]+1):cumsum(c(0,modellags))[i+1])];
-        vt[is.na(vt[1:maxlag,i]),i] <- rep(rev(vt[(maxlag - modellags + 1)[i]:maxlag,i]),
-                                           ceiling((maxlag - modellags + 1) / modellags)[i])[is.na(vt[1:maxlag,i])];
-    }
-    matvt[1:maxlag,] <- vt;
-
 ##### Initialise ges #####
 ElementsGES <- function(C){
     n.coef <- 0;
     if(measurementEstimate){
-        matw <- matrix(C[n.coef+(1:n.components)],1,n.components);
-        n.coef <- n.coef + n.components;
+        matw <- matrix(C[n.coef+(1:nComponents)],1,nComponents);
+        n.coef <- n.coef + nComponents;
     }
     else{
-        matw <- matrix(measurement,1,n.components);
+        matw <- matrix(measurement,1,nComponents);
     }
 
     if(transitionEstimate){
-        matF <- matrix(C[n.coef+(1:(n.components^2))],n.components,n.components);
-        n.coef <- n.coef + n.components^2;
+        matF <- matrix(C[n.coef+(1:(nComponents^2))],nComponents,nComponents);
+        n.coef <- n.coef + nComponents^2;
     }
     else{
-        matF <- matrix(transition,n.components,n.components);
+        matF <- matrix(transition,nComponents,nComponents);
     }
 
     if(persistenceEstimate){
-        vecg <- matrix(C[n.coef+(1:n.components)],n.components,1);
-        n.coef <- n.coef + n.components;
+        vecg <- matrix(C[n.coef+(1:nComponents)],nComponents,1);
+        n.coef <- n.coef + nComponents;
     }
     else{
-        vecg <- matrix(persistence,n.components,1);
+        vecg <- matrix(persistence,nComponents,1);
     }
 
-    vt <- matrix(NA,maxlag,n.components);
+    vt <- matrix(NA,maxlag,nComponents);
     if(initialType!="b"){
         if(initialType=="o"){
             vtvalues <- C[n.coef+(1:(orders %*% lags))];
             n.coef <- n.coef + orders %*% lags;
 
-            for(i in 1:n.components){
+            for(i in 1:nComponents){
                 vt[(maxlag - modellags + 1)[i]:maxlag,i] <- vtvalues[((cumsum(c(0,modellags))[i]+1):cumsum(c(0,modellags))[i+1])];
                 vt[is.na(vt[1:maxlag,i]),i] <- rep(rev(vt[(maxlag - modellags + 1)[i]:maxlag,i]),
                                                    ceiling((maxlag - modellags + 1) / modellags)[i])[is.na(vt[1:maxlag,i])];
@@ -162,31 +103,31 @@ ElementsGES <- function(C){
         }
     }
     else{
-        vt[,] <- matvt[1:maxlag,n.components];
+        vt[,] <- matvt[1:maxlag,nComponents];
     }
 
 # If exogenous are included
     if(xregEstimate){
-        at <- matrix(NA,maxlag,n.exovars);
+        at <- matrix(NA,maxlag,nExovars);
         if(initialXEstimate){
-            at[,] <- rep(C[n.coef+(1:n.exovars)],each=maxlag);
-            n.coef <- n.coef + n.exovars;
+            at[,] <- rep(C[n.coef+(1:nExovars)],each=maxlag);
+            n.coef <- n.coef + nExovars;
         }
         else{
             at <- matat[1:maxlag,];
         }
         if(FXEstimate){
-            matFX <- matrix(C[n.coef+(1:(n.exovars^2))],n.exovars,n.exovars);
-            n.coef <- n.coef + n.exovars^2;
+            matFX <- matrix(C[n.coef+(1:(nExovars^2))],nExovars,nExovars);
+            n.coef <- n.coef + nExovars^2;
         }
 
         if(gXEstimate){
-            vecgX <- matrix(C[n.coef+(1:n.exovars)],n.exovars,1);
-            n.coef <- n.coef + n.exovars;
+            vecgX <- matrix(C[n.coef+(1:nExovars)],nExovars,1);
+            n.coef <- n.coef + nExovars;
         }
     }
     else{
-        at <- matrix(0,maxlag,n.exovars);
+        at <- matrix(matat[1:maxlag,],maxlag,nExovars);
     }
 
     return(list(matw=matw,matF=matF,vecg=vecg,vt=vt,at=at,matFX=matFX,vecgX=vecgX));
@@ -221,7 +162,7 @@ CreatorGES <- function(silentText=FALSE,...){
     environment(ICFunction) <- environment();
 
 # 1 stands for the variance
-    n.param <- 2*n.components + n.components^2 + orders %*% lags * (initialType!="b") + !is.null(xreg) * n.exovars + (updateX)*(n.exovars^2 + n.exovars) + 1;
+    nParam <- 2*nComponents + nComponents^2 + orders %*% lags * (initialType!="b") + (!is.null(xreg)) * nExovars + (updateX)*(nExovars^2 + nExovars) + 1;
 
 # If there is something to optimise, let's do it.
     if(any((initialType=="o"),(measurementEstimate),(transitionEstimate),(persistenceEstimate),
@@ -230,16 +171,16 @@ CreatorGES <- function(silentText=FALSE,...){
         C <- NULL;
 # matw, matF, vecg, vt
         if(measurementEstimate){
-            C <- c(C,rep(1,n.components));
+            C <- c(C,rep(1,nComponents));
         }
         if(transitionEstimate){
             #C <- c(C,as.vector(test$transition));
-            #C <- c(C,rep(1,n.components^2 - length(test$transition)))
-            C <- c(C,rep(1,n.components^2));
-            #C <- c(C,c(diag(1,n.components)));
+            #C <- c(C,rep(1,nComponents^2 - length(test$transition)))
+            C <- c(C,rep(1,nComponents^2));
+            #C <- c(C,c(diag(1,nComponents)));
         }
         if(persistenceEstimate){
-            C <- c(C,rep(0.1,n.components));
+            C <- c(C,rep(0.1,nComponents));
         }
         if(initialType=="o"){
             C <- c(C,intercept);
@@ -258,10 +199,10 @@ CreatorGES <- function(silentText=FALSE,...){
             }
             if(updateX){
                 if(FXEstimate){
-                    C <- c(C,c(diag(n.exovars)));
+                    C <- c(C,c(diag(nExovars)));
                 }
                 if(gXEstimate){
-                    C <- c(C,rep(0,n.exovars));
+                    C <- c(C,rep(0,nExovars));
                 }
             }
         }
@@ -297,18 +238,97 @@ CreatorGES <- function(silentText=FALSE,...){
         cfType <- "MSE";
     }
 
-    ICValues <- ICFunction(n.param=n.param+n.param.intermittent,C=C,Etype=Etype);
+    ICValues <- ICFunction(nParam=nParam+nParamIntermittent,C=C,Etype=Etype);
     ICs <- ICValues$ICs;
     logLik <- ICValues$llikelihood;
 
-    icBest <- ICs["AICc"];
+    icBest <- ICs[ic];
 
 # Revert to the provided cost function
     cfType <- cfTypeOriginal
 
-    return(list(cfObjective=cfObjective,C=C,ICs=ICs,icBest=icBest,n.param=n.param,logLik=logLik));
+    return(list(cfObjective=cfObjective,C=C,ICs=ICs,icBest=icBest,nParam=nParam,logLik=logLik));
 }
 
+##### Preset y.fit, y.for, errors and basic parameters #####
+    matvt <- matrix(NA,nrow=obsStates,ncol=nComponents);
+    y.fit <- rep(NA,obsInsample);
+    y.for <- rep(NA,h);
+    errors <- rep(NA,obsInsample);
+
+##### Prepare exogenous variables #####
+    xregdata <- ssXreg(data=data, xreg=xreg, updateX=updateX,
+                       persistenceX=persistenceX, transitionX=transitionX, initialX=initialX,
+                       obsInsample=obsInsample, obsAll=obsAll, obsStates=obsStates, maxlag=maxlag, h=h, silent=silentText);
+
+    if(xregDo=="u"){
+        nExovars <- xregdata$nExovars;
+        matxt <- xregdata$matxt;
+        matat <- xregdata$matat;
+        xregEstimate <- xregdata$xregEstimate;
+        matFX <- xregdata$matFX;
+        vecgX <- xregdata$vecgX;
+        xregNames <- colnames(matxt);
+    }
+    else{
+        nExovars <- 1;
+        nExovarsOriginal <- xregdata$nExovars;
+        matxtOriginal <- xregdata$matxt;
+        matatOriginal <- xregdata$matat;
+        xregEstimateOriginal <- xregdata$xregEstimate;
+        matFXOriginal <- xregdata$matFX;
+        vecgXOriginal <- xregdata$vecgX;
+
+        matxt <- matrix(1,nrow(matxtOriginal),1);
+        matat <- matrix(0,nrow(matatOriginal),1);
+        xregEstimate <- FALSE;
+        matFX <- matrix(1,1,1);
+        vecgX <- matrix(0,1,1);
+        xregNames <- NULL;
+    }
+    xreg <- xregdata$xreg;
+    FXEstimate <- xregdata$FXEstimate;
+    gXEstimate <- xregdata$gXEstimate;
+    initialXEstimate <- xregdata$initialXEstimate;
+
+    # These three are needed in order to use ssgeneralfun.cpp functions
+    Etype <- "A";
+    Ttype <- "N";
+    Stype <- "N";
+
+# Check number of parameters vs data
+    nParamExo <- FXEstimate*length(matFX) + gXEstimate*nrow(vecgX) + initialXEstimate*ncol(matat);
+    nParamMax <- nParamMax + nParamExo + (intermittent!="n");
+
+##### Check number of observations vs number of max parameters #####
+    if(obsNonzero <= nParamMax){
+        if(!silentText){
+            message(paste0("Number of non-zero observations is ",obsNonzero,
+                           ", while the number of parameters to estimate is ", nParamMax,"."));
+        }
+        stop("Not enough observations. Can't fit the model you ask.",call.=FALSE);
+    }
+
+##### Preset values of matvt ######
+    slope <- cov(yot[1:min(12,obsNonzero),],c(1:min(12,obsNonzero)))/var(c(1:min(12,obsNonzero)));
+    intercept <- sum(yot[1:min(12,obsNonzero),])/min(12,obsNonzero) - slope * (sum(c(1:min(12,obsNonzero)))/min(12,obsNonzero) - 1);
+
+    vtvalues <- intercept;
+    if((orders %*% lags)>1){
+        vtvalues <- c(vtvalues,slope);
+    }
+    if((orders %*% lags)>2){
+        vtvalues <- c(vtvalues,yot[1:(orders %*% lags-2),]);
+    }
+
+    vt <- matrix(NA,maxlag,nComponents);
+    for(i in 1:nComponents){
+        vt[(maxlag - modellags + 1)[i]:maxlag,i] <- vtvalues[((cumsum(c(0,modellags))[i]+1):cumsum(c(0,modellags))[i+1])];
+        vt[is.na(vt[1:maxlag,i]),i] <- rep(rev(vt[(maxlag - modellags + 1)[i]:maxlag,i]),
+                                           ceiling((maxlag - modellags + 1) / modellags)[i])[is.na(vt[1:maxlag,i])];
+    }
+    matvt[1:maxlag,] <- vt;
+
 ##### Start the calculations #####
     environment(intermittentParametersSetter) <- environment();
     environment(intermittentMaker) <- environment();
@@ -371,6 +391,58 @@ CreatorGES <- function(silentText=FALSE,...){
 
     list2env(gesValues,environment());
 
+    if(xregDo!="u"){
+# Prepare for fitting
+        elements <- ElementsGES(C);
+        matw <- elements$matw;
+        matF <- elements$matF;
+        vecg <- elements$vecg;
+        matvt[1:maxlag,] <- elements$vt;
+        matat[1:maxlag,] <- elements$at;
+        matFX <- elements$matFX;
+        vecgX <- elements$vecgX;
+
+        ssFitter(ParentEnvironment=environment());
+
+        xregNames <- colnames(matxtOriginal);
+        xregNew <- cbind(errors,xreg[1:nrow(errors),]);
+        colnames(xregNew)[1] <- "errors";
+        colnames(xregNew)[-1] <- xregNames;
+        xregNew <- as.data.frame(xregNew);
+        xregResults <- stepwise(xregNew, ic=ic, silent=TRUE, df=nParam+nParamIntermittent-1);
+        xregNames <- names(coef(xregResults))[-1];
+        nExovars <- length(xregNames);
+        if(nExovars>0){
+            xregEstimate <- TRUE;
+            matxt <- as.data.frame(matxtOriginal)[,xregNames];
+            matat <- as.data.frame(matatOriginal)[,xregNames];
+            matFX <- diag(nExovars);
+            vecgX <- matrix(0,nExovars,1);
+
+            if(nExovars==1){
+                matxt <- matrix(matxt,ncol=1);
+                matat <- matrix(matat,ncol=1);
+                colnames(matxt) <- colnames(matat) <- xregNames;
+            }
+            else{
+                matxt <- as.matrix(matxt);
+                matat <- as.matrix(matat);
+            }
+        }
+        else{
+            nExovars <- 1;
+            xreg <- NULL;
+        }
+
+        if(!is.null(xreg)){
+            gesValues <- CreatorGES(silentText=TRUE);
+            list2env(gesValues,environment());
+        }
+    }
+
+    if(!is.null(xreg)){
+        xreg <- matxt[,xregNames];
+    }
 # Prepare for fitting
     elements <- ElementsGES(C);
     matw <- elements$matw;
@@ -423,14 +495,14 @@ CreatorGES <- function(silentText=FALSE,...){
     matvt <- ts(matvt,start=(time(data)[1] - deltat(data)*maxlag),frequency=frequency(data));
     if(!is.null(xreg)){
         matvt <- cbind(matvt,matat);
-        colnames(matvt) <- c(paste0("Component ",c(1:n.components)),colnames(matat));
+        colnames(matvt) <- c(paste0("Component ",c(1:nComponents)),colnames(matat));
         if(updateX){
             rownames(vecgX) <- xregNames;
             dimnames(matFX) <- list(xregNames,xregNames);
         }
     }
     else{
-        colnames(matvt) <- paste0("Component ",c(1:n.components));
+        colnames(matvt) <- paste0("Component ",c(1:nComponents));
     }
 
     if(holdout==T){
@@ -442,7 +514,13 @@ CreatorGES <- function(silentText=FALSE,...){
         errormeasures <- NA;
     }
 
-    modelname <- paste0("GES(",paste(orders,"[",lags,"]",collapse=",",sep=""),")");
+    if(!is.null(xreg)){
+        modelname <- "GESX";
+    }
+    else{
+        modelname <- "GES";
+    }
+    modelname <- paste0(modelname,"(",paste(orders,"[",lags,"]",collapse=",",sep=""),")");
     if(all(intermittent!=c("n","none"))){
         modelname <- paste0("i",modelname);
     }
@@ -476,7 +554,7 @@ CreatorGES <- function(silentText=FALSE,...){
     model <- list(model=modelname,timeElapsed=Sys.time()-startTime,
                   states=matvt,measurement=matw,transition=matF,persistence=vecg,
                   initialType=initialType,initial=initialValue,
-                  nParam=n.param,
+                  nParam=nParam,
                   fitted=y.fit,forecast=y.for,lower=y.low,upper=y.high,residuals=errors,
                   errors=errors.mat,s2=s2,intervals=intervalsType,level=level,
                   actuals=data,holdout=y.holdout,iprob=pt,intermittent=intermittent,
