@@ -323,12 +323,13 @@ elementsGenerator <- function(ar.orders=ar.orders, ma.orders=ma.orders, i.orders
         }
         matot <- matrix(NA,obs,nsim);
         # Generate values for occurence variable
-        if(all(iprob < 1) & all(iprob > 0)){
-            matot[,] <- rbinom(obs*nsim,1,iprob);
-        }
-        else{
+        if(all(iprob == 1)){
             matot[,] <- 1;
         }
+        else{
+            matot[,] <- rbinom(obs*nsim,1,iprob);
+        }
+
         matot <- ts(matot,frequency=frequency);
         materrors <- ts(materrors,frequency=frequency);
         matyt <- materrors;
@@ -492,21 +493,21 @@ elementsGenerator <- function(ar.orders=ar.orders, ma.orders=ma.orders, i.orders
     veclikelihood <- -obs/2 *(log(2*pi*exp(1)) + log(colMeans(materrors^2)));
 
 # Generate ones for the possible intermittency
-    if(all(iprob < 1) & all(iprob > 0)){
-        matot[,] <- rbinom(obs*nsim,1,iprob);
+    if(all(iprob == 1)){
+        matot[,] <- 1;
     }
     else{
-        matot[,] <- 1;
+        matot[,] <- rbinom(obs*nsim,1,iprob);
     }
 
 #### Simulate the data ####
     simulateddata <- simulatorwrap(arrvt,materrors,matot,arrF,matw,matg,"A","N","N",modellags);
 
-    if(all(iprob < 1) & all(iprob > 0)){
-        matyt <- round(simulateddata$matyt,0);
+    if(all(iprob == 1)){
+        matyt <- simulateddata$matyt;
     }
     else{
-        matyt <- simulateddata$matyt;
+        matyt <- round(simulateddata$matyt,0);
     }
     arrvt <- simulateddata$arrvt;
     dimnames(arrvt) <- list(NULL,componentsNames,NULL);
@@ -515,7 +516,6 @@ elementsGenerator <- function(ar.orders=ar.orders, ma.orders=ma.orders, i.orders
         matInitialValue[,] <- arrvt[burnInPeriod+1,,];
         arrvtDim <- dim(arrvt);
         arrvtDim[1] <- arrvtDim[1] - burnInPeriod;
-        print(arrvtDim)
         arrvt <- array(arrvt[-c(1:burnInPeriod),,],arrvtDim);
         materrors <- materrors[-c(1:burnInPeriod),];
         matyt <- matyt[-c(1:burnInPeriod),];
@@ -567,6 +567,6 @@ elementsGenerator <- function(ar.orders=ar.orders, ma.orders=ma.orders, i.orders
     model <- list(model=modelname,
                   AR=matARValue, MA=matMAValue, constant=vecConstantValue, initial=matInitialValue,
                   data=matyt, states=arrvt, residuals=materrors,
-                  occurrences=matot, likelihood=veclikelihood);
+                  occurrences=matot, logLik=veclikelihood);
     return(structure(model,class="smooth.sim"));
 }
