@@ -18,7 +18,7 @@ test_that("Test on N1234$x, predefined ETS", {
 # Test combinations of ETS
 testModel <- es(Mcomp::M3$N2568$x, "CCC", silent=TRUE, ic="BIC");
 test_that("Test ETS(CCC) with BIC on N2568$x", {
-    expect_equal(round(testModel$s2,4), 0.0004);
+    expect_equal(testModel$s2, mean(testModel$residuals^2));
 })
 
 # Test model selection of non-multiplicative trend ETS
@@ -29,8 +29,8 @@ test_that("Test ETS(MXM) with AIC on N2568$x", {
 
 # Test trace cost function for ETS
 testModel <- es(Mcomp::M3$N2568$x, model="MAdM", h=18, holdout=TRUE, silent=TRUE, intervals=TRUE)
-test_that("Test AIC of ETS based on MSTFE on N2568$x", {
-    expect_equal(round(AIC(testModel),2), as.numeric(round(testModel$ICs["AIC"],2)));
+test_that("Test AIC of ETS on N2568$x", {
+    expect_equal(as.numeric(round(AIC(testModel),2)), as.numeric(round(testModel$ICs["AIC"],2)));
 })
 
 # Test how different passed values are accepted by ETS
@@ -50,6 +50,18 @@ test_that("Check exogenous variables for ETS on N1457", {
     expect_equal(suppressWarnings(forecast(testModel, h=18, holdout=FALSE)$model), testModel$model);
 })
 
+# Test selection of exogenous with ETS
+testModel <- es(y, h=18, holdout=TRUE, xreg=x, silent=TRUE, xregDo="select")
+test_that("Select exogenous variables for ETS on N1457", {
+    expect_equal(sum(testModel$xreg),1);
+})
+
+# Test combination of ETS with exogenous selection
+testModel <- es(y, "CCC", h=18, holdout=TRUE, xreg=x, silent=TRUE, xregDo="select")
+test_that("Select exogenous variables for CES on N1457", {
+    expect_match(testModel$model, "ETSX");
+})
+
 # iETS test
 x <- c(0,1,2,0,0,0,1,0,0,1,0,0,0,2,0,0,0,1,0,0);
 testModel <- es(x, "MNN", intermittent="a", silent=TRUE, ic="AIC");
